/*
   Copyright (c) 2014-2016 by Plexim GmbH
   All rights reserved.

   A free license is granted to anyone to use this software for any legal
   non safety-critical purpose, including commercial applications, provided
   that:
   1) IT IS NOT USED TO DIRECTLY OR INDIRECTLY COMPETE WITH PLEXIM, and
   2) THIS COPYRIGHT NOTICE IS PRESERVED in its entirety.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
 */

#ifndef PWR_IMPL_H_
#define PWR_IMPL_H_

#include "F28004x_Gpio_defines.h"

#define PWR_MAX_PWM_CHANNELS 8

typedef enum
{
	PWR_STATE_POWERUP,
	PWR_STATE_DISABLED,
	PWR_STATE_ENABLING,
	PWR_STATE_ENABLED,
	PWR_STATE_FAULT,
	PWR_STATE_FAULT_ACKN,
	PWR_STATE_CRITICAL_FAULT
} PWR_FsmState_t;

typedef enum
{
	PWR_ERR_NONE,
	PWR_GDRV_ERROR,
	PWR_ERR_UNKNOWN
} PWR_Error_t;

typedef struct PWR_OBJ
{
    GDRV_Handle_t gdrvHandle;
    uint16_t timer;
    uint16_t fsmExecRateHz;

    uint16_t numRegisteredPwmChannels;
    PWM_Handle_t pwmChannels[PWR_MAX_PWM_CHANNELS];

    uint16_t enableSwitchingReq;
    PIL_READ_PROBE(int16_t, pilMode, 0, 1, "");
    PIL_READ_PROBE(int16_t, state, 0, 1, "");
    PIL_READ_PROBE(uint16_t, gdrvInitReq, 0, 1, "");
    PIL_READ_PROBE(int16_t, gdrvStatus, 0, 1, "");
    PIL_READ_PROBE(uint16_t, enableReq, 0, 1, "");
    PIL_READ_PROBE(uint16_t, errorCode, 0, 1, "");
    PIL_READ_PROBE(uint16_t, gatesActive, 0, 1, "");
} PWR_Obj_t;

typedef PWR_Obj_t *PWR_Handle_t;
extern PWR_Handle_t PWR_SHandle;

inline void PWR_syncEnableSwitching()
{
    PWR_Obj_t *obj = (PWR_Obj_t *)PWR_SHandle;
    if(obj->enableSwitchingReq)
    {
        if(obj->pilMode == false)
        {
         // enable actuators
            int i;
            for(i=0; i< obj->numRegisteredPwmChannels; i++)
            {
                PWM_enableOut(obj->pwmChannels[i]);
            }
        }
        obj->gatesActive = true;
        obj->enableSwitchingReq = false;
    }
}

inline void PWR_configureTZGpio(uint16_t aTzId, uint16_t aGpio)
{
    // determine registers and masks
    uint16_t group = aGpio / 32;
    uint16_t groupIndex = aGpio % 32; // pin32
    uint16_t mux = groupIndex / 16;
    uint16_t muxIndex = groupIndex % 16; // pin 16
    uint32_t mask = 1L << groupIndex;
    uint32_t muxMask = 3L << (muxIndex * 2);

    uint32_t *ctrlRegsBaseAddr = (uint32_t *)&GpioCtrlRegs + group*GPY_CTRL_OFFSET;
    uint32_t *dirReg = ctrlRegsBaseAddr + GPYDIR;
    uint32_t *muxReg = ctrlRegsBaseAddr + GPYMUX + mux;
    uint32_t *muxGReg = ctrlRegsBaseAddr + GPYGMUX + mux;

    EALLOW;
    // configure hardware
    *muxReg &= ~muxMask;
    *muxGReg &= ~muxMask;
    *dirReg &= mask;

    switch(aTzId)
    {
        case 1:
            InputXbarRegs.INPUT1SELECT = aGpio;
            break;
        case 2:
            InputXbarRegs.INPUT2SELECT = aGpio;
            break;
        case 3:
            InputXbarRegs.INPUT3SELECT = aGpio;
            break;
        default:
            break;
    }

    EDIS;
}

#endif /* PWR_IMPL_H_ */
