/*
   Copyright (c) 2014-2016 by Plexim GmbH
   All rights reserved.

   A free license is granted to anyone to use this software for any legal
   non safety-critical purpose, including commercial applications, provided
   that:
   1) IT IS NOT USED TO DIRECTLY OR INDIRECTLY COMPETE WITH PLEXIM, and
   2) THIS COPYRIGHT NOTICE IS PRESERVED in its entirety.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
 */

#ifndef SPI_IMPL_H_
#define SPI_IMPL_H_

/* TI convention:
 * pol/phase = 0/0: inactive level low, slave read at falling edge (transitions at rising edge)
 * pol/phase = 0/1: inactive level low, slave read at rising edge (transitions at falling edge)
 * pol/phase = 1/0: inactive level high, slave read at rising edge (transitions at falling edge)
 * pol/phase = 1/1: inactive level high, slave read at falling edge (transitions at rising edge)
 */

typedef struct SPI_PARAMS {
    Uint16              SPICHAR;   // Character Length
    bool              CLKPOLARITY;    // Shift Clock Polarity
    bool               CLKPHASE;     // SPI Clock Phase Select
    Uint32              BAUDRATE; //SPI Baud Rate Control
    bool                MASTERSLAVE; // Slave-0 Master-1
 } SPI_Params_t;

typedef struct SPI_OBJ {
	uint16_t port;
	uint32_t portHandle;
	uint32_t clk;
} SPI_Obj_t;

typedef SPI_Obj_t *SPI_Handle_t;

inline void SPI_sinit(){}

#define SPI_REGS_PTR ((volatile struct SPI_REGS *)obj->portHandle)

inline void SPI_setMasterOut(SPI_Handle_t aHandle, uint16_t aChar)
{
    SPI_Obj_t *obj = (SPI_Obj_t *)aHandle;
    SPI_REGS_PTR->SPITXBUF = aChar;
}

inline uint16_t SPI_getMasterIn(SPI_Handle_t aHandle)
{
    SPI_Obj_t *obj = (SPI_Obj_t *)aHandle;
    return SPI_REGS_PTR->SPIRXBUF;
}

inline uint16_t SPI_getSlaveIn(SPI_Handle_t aHandle)
{
    SPI_Obj_t *obj = (SPI_Obj_t *)aHandle;
    return SPI_REGS_PTR->SPIRXBUF;
}

inline void SPI_setSlaveOut(SPI_Handle_t aHandle, uint16_t aChar)
{
    SPI_Obj_t *obj = (SPI_Obj_t *)aHandle;
    SPI_REGS_PTR->SPITXBUF = aChar;
}

inline uint16_t SPI_getTxFifoLevel(SPI_Handle_t aHandle)
{
    SPI_Obj_t *obj = (SPI_Obj_t *)aHandle;
    return SPI_REGS_PTR->SPIFFTX.bit.TXFFST;
}

inline uint16_t SPI_getRxFifoLevel(SPI_Handle_t aHandle)
{
    SPI_Obj_t *obj = (SPI_Obj_t *)aHandle;
    return SPI_REGS_PTR->SPIFFRX.bit.RXFFST;
}

inline void SPI_resetFifos(SPI_Handle_t aHandle)
{
    SPI_Obj_t *obj = (SPI_Obj_t *)aHandle;

    SPI_REGS_PTR->SPIFFRX.bit.RXFIFORESET = 0;
    SPI_REGS_PTR->SPIFFTX.bit.TXFIFO = 0;
    SPI_REGS_PTR->SPIFFRX.bit.RXFIFORESET = 1;
    SPI_REGS_PTR->SPIFFTX.bit.TXFIFO = 1;
}

inline void SPI_putWords(SPI_Handle_t aHandle, uint16_t *aData, uint16_t aLen){
    SPI_Obj_t *obj = (SPI_Obj_t *)aHandle;

    int i;
    for(i=0; i<aLen; i++)
    {
        SPI_REGS_PTR->SPITXBUF = aData[i];
    }
}

inline void SPI_getWords(SPI_Handle_t aHandle, uint16_t *aData, uint16_t aLen){
    SPI_Obj_t *obj = (SPI_Obj_t *)aHandle;

    int i;
    for(i=0; i<aLen; i++)
    {
        aData[i] = SPI_REGS_PTR->SPIRXBUF;
    }
}

#endif /* SPI_IMPL_H_ */
