/*
   Copyright (c) 2014-2019 by Plexim GmbH
   All rights reserved.

   A free license is granted to anyone to use this software for any legal
   non safety-critical purpose, including commercial applications, provided
   that:
   1) IT IS NOT USED TO DIRECTLY OR INDIRECTLY COMPETE WITH PLEXIM, and
   2) THIS COPYRIGHT NOTICE IS PRESERVED in its entirety.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
 */

#include "power.h"

#ifndef HAL_IMPL_H_
#define HAL_IMPL_H_

#ifndef HAL_NUM_TZ_CHANNELS
#define HAL_NUM_TZ_CHANNELS 3
#endif

#define HAL_NUM_CHANNELS_PER_ADC AIN_NUM_CHANNELS
#define HAL_VOLTS_PER_ADC_BIT (3.0/4096.0) //VREFHI 3.0V for 2837xS

typedef struct HAL_SOBJ
{
#ifndef PIL_PREP_TOOL
	DIO_Handle_t inPortHandles[HAL_NUM_DIN_CHANNELS];
	DIO_Handle_t outPortHandles[HAL_NUM_DOUT_CHANNELS];
	PWM_Handle_t pwmHandles[HAL_NUM_PWM_CHANNELS];
	QEP_Handle_t qepHandles[HAL_NUM_QEP_CHANNELS];
	AIN_Handle_t aInHandle[HAL_NUM_ADCS];
	SPI_Handle_t spiHandles[HAL_NUM_SPI_CHANNELS];

	DIO_Obj_t inPortObjs[HAL_NUM_DIN_CHANNELS];
	DIO_Obj_t outPortObjs[HAL_NUM_DOUT_CHANNELS];
	PWM_Obj_t pwmObjs[HAL_NUM_PWM_CHANNELS];
	QEP_Obj_t qepObjs[HAL_NUM_QEP_CHANNELS];
	AIN_Obj_t aInObjs[HAL_NUM_ADCS];
	SPI_Obj_t spiObjs[HAL_NUM_SPI_CHANNELS];
#endif
	uint32_t sysClkHz;
	float aInScale[HAL_NUM_ADCS][HAL_NUM_CHANNELS_PER_ADC];
	float aInOffset[HAL_NUM_ADCS][HAL_NUM_CHANNELS_PER_ADC];
	AIN_AdcParams_t aInParamsMemory;
	AIN_ChannelParams_t aInChannelParamsMemory;
	PWM_Params_t pwmParamsMemory;
    	QEP_Params_t qepParamsMemory;
    	SPI_Params_t spiParamsMemory;
} HAL_SObj_t;

extern HAL_SObj_t HAL_SObj;

inline uint16_t HAL_getAnalogIn(uint16_t aHandle, uint16_t aChannel)
{
	return AIN_getIn(HAL_SObj.aInHandle[aHandle], aChannel);
}

inline float HAL_getAnalogInF(uint16_t aAdc, uint16_t aChannel)
{
    float adcV = (float)AIN_getIn(HAL_SObj.aInHandle[aAdc], aChannel)*HAL_VOLTS_PER_ADC_BIT;
    return (adcV * HAL_SObj.aInScale[aAdc][aChannel]) + HAL_SObj.aInOffset[aAdc][aChannel];
}

inline bool HAL_getDigitalIn(uint16_t aChannel)
{
	return DIO_get(HAL_SObj.inPortHandles[aChannel]);
}

inline void HAL_setDigitalOut(uint16_t aChannel, bool aVal)
{
	DIO_set(HAL_SObj.outPortHandles[aChannel], aVal);
}

inline uint32_t HAL_getPwmPeriod(uint16_t aChannel)
{
	return PWM_getPeriod(HAL_SObj.pwmHandles[aChannel]);
}

inline uint32_t HAL_getPwmCounter(uint16_t aChannel)
{
	return PWM_getCounter(HAL_SObj.pwmHandles[aChannel]);
}

inline bool HAL_getPwmCountDirection(uint16_t aChannel)
{
	return PWM_getCountDirection(HAL_SObj.pwmHandles[aChannel]);
}

inline void HAL_setPwm(uint16_t aChannel, uint16_t aVal)
{
	PWM_setCompare(HAL_SObj.pwmHandles[aChannel], aVal);
}

// deprecated
inline void HAL_setPwmF(uint16_t aChannel, float aDuty)
{
    if(aDuty < 0)
    {
        aDuty = 0;
    }
    else if (aDuty > 1.0)
    {
        aDuty = 1.0;
    }
    uint16_t duty = (uint16_t)((float)HAL_getPwmPeriod(aChannel) * (1.0-aDuty));
    HAL_setPwm(aChannel, duty);
}

inline void HAL_setPwmDuty(uint16_t aChannel, float aDuty)
{
    if(aDuty < 0)
    {
        aDuty = 0;
    }
    else if (aDuty > 1.0)
    {
        aDuty = 1.0;
    }
    uint16_t duty = (uint16_t)(aDuty * (float)HAL_getPwmPeriod(aChannel));
    HAL_setPwm(aChannel, duty);
}

inline uint32_t HAL_getQepCounter(uint16_t aChannel)
{
	return QEP_getPosCnt(HAL_SObj.qepHandles[aChannel]);
}

inline bool HAL_getAndClearQepIndexFlag(uint16_t aChannel)
{
	return QEP_getAndClearIndexFlag(HAL_SObj.qepHandles[aChannel]);
}

inline uint32_t HAL_getQepIndexLatchCounter(uint16_t aChannel)
{
	return QEP_getPosILatchCnt(HAL_SObj.qepHandles[aChannel]);
}

inline void HAL_setSpiMasterOut(uint16_t aChannel, uint16_t aChar)
{
    SPI_setMasterOut(HAL_SObj.spiHandles[aChannel], aChar);
}

inline uint16_t HAL_getSpiMasterIn(uint16_t aChannel)
{
    return SPI_getMasterIn(HAL_SObj.spiHandles[aChannel]);
}

inline uint16_t HAL_getSpiSlaveIn(uint16_t aChannel)
{
    return SPI_getSlaveIn(HAL_SObj.spiHandles[aChannel]);
}

inline void HAL_setSpiSlaveOut(uint16_t aChannel, uint16_t aChar)
{
    SPI_setSlaveOut(HAL_SObj.spiHandles[aChannel], aChar);
}

inline uint16_t HAL_getSpiTxFifoLevel(int16_t aChannel)
{
    return SPI_getTxFifoLevel(HAL_SObj.spiHandles[aChannel]);
}

inline uint16_t HAL_getSpiRxFifoLevel(int16_t aChannel)
{
    return SPI_getRxFifoLevel(HAL_SObj.spiHandles[aChannel]);
}

inline void HAL_resetSpiFifos(int16_t aChannel)
{
    SPI_resetFifos(HAL_SObj.spiHandles[aChannel]);
}

inline void HAL_putSpiWords(int16_t aChannel, uint16_t *aData, uint16_t aLen)
{
    SPI_putWords(HAL_SObj.spiHandles[aChannel], aData, aLen);
}

inline void HAL_getSpiWords(int16_t aChannel, uint16_t *aData, uint16_t aLen)
{
    SPI_getWords(HAL_SObj.spiHandles[aChannel], aData, aLen);
}

inline void HAL_putGetSpiWords(int16_t aChannel, uint16_t *aDataOut, uint16_t *aDataIn, uint16_t aLen)
{
    SPI_putGetWords(HAL_SObj.spiHandles[aChannel], aDataOut, aDataIn, aLen);
}

inline void HAL_setPwmPhase(uint16_t aChannel, uint16_t aVal, uint16_t aPh, uint16_t aDir)
{
	PWM_setComparePh(HAL_SObj.pwmHandles[aChannel], aVal, aPh, aDir);
}

inline void HAL_setPwmPh(uint16_t aChannel, float aDuty, float aPhase, uint16_t aCntrMode)
{
	if(aDuty < 0)
	{
		aDuty = 0;
	}
	else if (aDuty > 1.0)
	{
		aDuty = 1.0;
	}
	uint16_t duty = (uint16_t)(aDuty * (float)HAL_getPwmPeriod(aChannel) );
	if(aPhase < 0)
	{
		aPhase = 0;
	}
	else if (aPhase > 1.0)
	{
		aPhase = 1.0;
	}

	uint16_t phase;
	uint16_t dir;
	switch (aCntrMode)
	{
		case 0:
			phase = (uint16_t)((float)HAL_getPwmPeriod(aChannel) * (1-aPhase));
			dir = 0;
			break;
		case 2:
			if (aPhase <= 0.5)
			{
				phase = (uint16_t)((float)HAL_getPwmPeriod(aChannel) * (aPhase) * 2);
				dir = 0;
			}
			else
			{
				phase = (uint16_t)((float)HAL_getPwmPeriod(aChannel) * (1-aPhase) * 2);
				dir = 1;
			}
			break;
		default:
			break;
	}
	HAL_setPwmPhase(aChannel, duty, phase, dir);
}

inline void HAL_enablePower(bool aEnable)
{
    PWR_enable(aEnable);
}

inline bool HAL_powerIsEnabled()
{
    return PWR_isEnabled();
}

#endif /* HAL_IMPL_H_ */
