local U = { }

math.randomseed(os.time())
local random = math.random

-- https://stackoverflow.com/questions/1340230/check-if-directory-exists-in-lua
function U.FileExists(file)
   if (Target.Variables.HOST_OS == "win") and (string.sub(file, 1, 1) == "/") then
       -- this check is necessary, because Lua will silently replace "/" with "C:/"
       return false
   end
   local ok, err, code = os.rename(file, file)
   if not ok then
      if (code == 13) or (code == 17) then
         -- Permission denied, but it exists
         return true
      end
   end
   return ok, err
end

function U.CopyTemplateFile(src, dest, subs)
  local file = io.open(src, "rb")
  local src_content = file:read("*all")
  io.close(file)
  local dest_content

  file = io.open(dest, "rb")
  if(file == nil) then
    dest_content = nil
  else
    dest_content = file:read("*all")
    io.close(file)
  end
  
  if subs ~= nil then
  	for _,v in pairs(subs) do 
  		local before = v["before"]
  		local after = v["after"]
  		src_content = string.gsub( src_content, before, after)  
	end
  end

  src_content = string.gsub(src_content, "\r", "") 
  
  if not (src_content == dest_content) then
    file = io.open(dest, "w")
    io.output(file)
    io.write(src_content)
    file.close() 
  end
end

function U.GetFromArrayOrScalar(field, index, majordim)
	if #field == 1 then
   	return field[1]
  	elseif #field == majordim then
  	 	return field[index]
  	else
    	return nil	
  	end  
end

function U.Round(num, numDecimalPlaces)
  if numDecimalPlaces and numDecimalPlaces>0 then
    local mult = 10^numDecimalPlaces
    return math.floor(num * mult + 0.5) / mult
  end
  return math.floor(num + 0.5)
end

function U.guid()
    local template ='{0xXX, 0xXX, 0xXX, 0xXX, 0xXX, 0xXX, 0xXX, 0xXX}'
    return string.gsub(template, '[X]', function (c)
        local v = (c == 'X') and random(0, 0xf) 
        return string.format('%x', v)
    end)
end

function U.IsValidCName(name)
	if string.match(name, '[^a-zA-Z0-9_]') ~= nil then
  		return false
	end
	if string.match(string.sub(name, 1, 1), '[0-9]') ~= nil then
  		return false
	end
	return true
end

return U
